@@ -53,18 +53,6 @@ initcpopen(void)
         return;
 }
 
-/* Just like close() but retries on interrupt */
-static int
-safeClose(int fd) {
-    int rv;
-
-    do {
-        rv = close(fd);
-    } while ((rv < 0) && (errno == EINTR));
-
-    return rv;
-}
-
 /* Closes all open FDs except for stdin, stdout and stderr */
 static void
 closeFDs(int errnofd) {
@@ -92,7 +80,7 @@ closeFDs(int errnofd) {
             continue;
         }
 
-        safeClose(fdNum);
+        close(fdNum);
     }
 
     /* Closes dp and the underlying dfd */
@@ -182,6 +170,28 @@ pyTryIntToLong(PyObject *intNum, int *gotIt) {
     return value;
 }
 
+static int
+unsetCloseOnExec(int fd) {
+    int flags;
+    flags = fcntl(fd, F_GETFD, 0);
+    return fcntl(fd, F_SETFD, flags & (~FD_CLOEXEC));
+}
+
+/* dup2 oldfd to newfd, disabling close-on-exec flag for newfd.
+ *
+ * dup2 unset newfd close-on-exec flag, but it does nothing if old fd is
+ * invalid, or when oldfd is equal to newfd.  In this case, if newfd
+ * close-on-exec flag was set, the fd will be closed after execv the child,
+ * which is not very useful for the child.
+ */
+static int
+dupFdForChild(int oldfd, int newfd) {
+    if (oldfd == -1 || oldfd == newfd)
+        return unsetCloseOnExec(newfd);
+    else
+        return dup2(oldfd, newfd);
+}
+
 /* Python's implementation of Popen forks back to python before execing.
  * Forking a python proc is a very complex and volatile process.
  *
@@ -250,34 +260,24 @@ createProcess(PyObject *self, PyObject *args)
         goto fail;
     }
 
-try_fork:
     cpid = fork();
     if (cpid < 0) {
-        if (errno == EAGAIN ||
-            errno == EINTR ) {
-            goto try_fork;
-        }
-
         PyErr_SetFromErrno(PyExc_OSError);
         goto fail;
     }
 
     if (!cpid) {
-        safeClose(0);
-        safeClose(1);
-        safeClose(2);
+        dupFdForChild(outfd[0], 0);
+        dupFdForChild(in1fd[1], 1);
+        dupFdForChild(in2fd[1], 2);
 
-        dup2(outfd[0], 0);
-        dup2(in1fd[1], 1);
-        dup2(in2fd[1], 2);
-
-        safeClose(outfd[0]);
-        safeClose(outfd[1]);
-        safeClose(in1fd[0]);
-        safeClose(in1fd[1]);
-        safeClose(in2fd[0]);
-        safeClose(in2fd[1]);
-        safeClose(errnofd[0]);
+        close(outfd[0]);
+        close(outfd[1]);
+        close(in1fd[0]);
+        close(in1fd[1]);
+        close(in2fd[0]);
+        close(in2fd[1]);
+        close(errnofd[0]);
 
         if (deathSignal) {
             childErrno = prctl(PR_SET_PDEATHSIG, deathSignal);
@@ -287,11 +287,11 @@ try_fork:
             /* Check that parent did not already die between fork and us
              * setting the death signal */
             if (write(errnofd[1], &childErrno, sizeof(int)) < sizeof(int)) {
-                exit(-1);
+                _exit(-1);
             }
 
             if (childErrno != 0) {
-                exit(-1);
+                _exit(-1);
             }
         }
 
@@ -316,26 +316,20 @@ try_fork:
             }
         }
 
-exec:
         if (envp) {
             execvpe(argv[0], argv, envp);
         } else {
             execvp(argv[0], argv);
         }
 
-        if (errno == EINTR ||
-            errno == EAGAIN )
-        {
-            goto exec;
-        }
 sendErrno:
         if (write(errnofd[1], &errno, sizeof(int)) < 0) {
-            exit(errno);
+            _exit(errno);
         }
-        exit(-1);
+        _exit(-1);
     }
 
-    safeClose(errnofd[1]);
+    close(errnofd[1]);
     errnofd[1] = -1;
 
     if (deathSignal) {
@@ -374,7 +368,7 @@ sendErrno:
         goto fail;
     }
 
-    safeClose(errnofd[0]);
+    close(errnofd[0]);
     errnofd[0] = -1;
 
     /* From this point errors shouldn't occur, if they do something is very
@@ -398,11 +392,11 @@ fail:
     }
 
     if (errnofd[0] >= 0) {
-        safeClose(errnofd[0]);
+        close(errnofd[0]);
     }
 
     if (errnofd[1] >= 0) {
-        safeClose(errnofd[1]);
+        close(errnofd[1]);
     }
 
     return NULL;
